----------------------------------------------------------------------
-- A toolbar to send commands to a Mega Drive running a Mega Everdrive x7 USB port.
----------------------------------------------------------------------

local function round(x)
  if x%2 ~= 0.5 then
    return math.floor(x + 0.5)
  end
  return x - 0.5
end

local function sendPalette(paletteIndex)
	-- let's just send the palette for now.
	local spr = app.activeSprite
	if not spr then
	  return app.alert("There is no active sprite")
	end
	
	local pal = spr.palettes[1]
	local command = "updatepalette " .. paletteIndex
	for i = 0, math.min(15, #pal-1) do
		command = command .. " " .. string.format("%d", round(pal:getColor(i).red / 36.428)) .. string.format("%d", round(pal:getColor(i).green / 36.428)) .. string.format("%d", round(pal:getColor(i).blue / 36.428))
	end
	if (#pal < 16) then
		for i = #pal, 15 do
			command = command .. " " .. "000"
		end
	end
	os.execute(os.getenv("MegaDriveDrawSocketAppLocation") .. "\\MegaDriveDrawSocketApp.exe " .. command)
end

local function getNearest8(input)
	local size = 8
	while (size < input) do
		size = size + 8
	end
	return size
end

local function checkSpriteSize(checkAnimation)
	local spr = app.activeSprite
	if not spr then
	  app.alert("There is no active sprite")
	  return false
	end
	
	if (spr.width > 320) then
	  app.alert("Sprite is too wide! Max width is 320 pixels.")
	  return false
	end
	
	if (spr.width > 160) then
	  app.alert("Sprite is too high! Max height is 160 pixels.")
	  return false
	end
	
	if not checkAnimation then return true end
	
	local nearestWidth8 = getNearest8(spr.width) / 8
	local nearestHeight8 = getNearest8(spr.height) / 8
	
	if (nearestWidth8 * nearestHeight8 * #spr.frames > 1800) then
	  app.alert("Sprite requires too much memory to render. Reduce animation length or sprite size to preview on the Mega Drive.")
	  return false
	end
	
	return true
end

local function sendScene(isBackground)
	-- check sprite sizes here
	if not checkSpriteSize(true) then return end
	
	-- send palette
	sendPalette(isBackground + 1)
	
	local spr = app.activeSprite
	local outputFilePath = os.getenv("MegaDriveDrawSocketAppLocation")
	local outputJSON = outputFilePath .. "\\TempOutput\\OutputSprite.json"
	app.command.ExportSpriteSheet{ type=SpriteSheetType.HORIZONTAL, rows=1, ui=false, askOverwrite=false, bestFit=true, dataFormat=SpriteSheetDataFormat.JSON_ARRAY, textureFilename=outputFilePath .. "\\TempOutput\\OutputSprite.png", dataFilename=outputJSON, trim=false, listLayers=false, listTags=false, listSlices=false, tag="" }
	os.execute(outputFilePath .. "\\MegaDriveDrawSocketApp.exe drawanimation " .. outputJSON .. " " .. isBackground .. " " .. (app.activeFrame.frameNumber - 1) .. " " .. #spr.frames)
end

local function sendFrame(isBackground)
	-- check sprite sizes here
	if not checkSpriteSize(false) then return end
	
	-- send palette
	sendPalette(isBackground + 1)
	
	local spr = app.activeSprite
	local outputFilePath = os.getenv("MegaDriveDrawSocketAppLocation")
	local outputPNG = outputFilePath .. "\\TempOutput\\OutputSprite.png"
	app.command.ExportSpriteSheet{ ui=false, askOverwrite=false, bestFit=true, dataFormat=SpriteSheetDataFormat.JSON_ARRAY, textureFilename=outputPNG, dataFilename=outputFilePath .. "\\TempOutput\\OutputSprite.json", trim=false, listLayers=false, listTags=false, listSlices=false, tag="**selected-frames**" }
	os.execute(outputFilePath .. "\\MegaDriveDrawSocketApp.exe drawsprite " .. outputPNG .. " " .. isBackground .. " " .. (app.activeFrame.frameNumber - 1) .. " " .. #spr.frames)
	
end

local function crushPalette()
	local spr = app.activeSprite
	if not spr then
	  return app.alert("There is no active sprite")
	end

	app.transaction(
	function()
		local pal = spr.palettes[1]
		for i = 0, #pal-1 do
			pal:setColor(i, Color{ r=round(round(pal:getColor(i).red / 36.428) * 36.428), g=round(round(pal:getColor(i).green / 36.428) * 36.428), b=round(round(pal:getColor(i).blue / 36.428) * 36.428), a=255 } )
		end
	end)
	app.refresh()
end

local function advanceFrame(background, forward)
	os.execute(os.getenv("MegaDriveDrawSocketAppLocation") .. "\\MegaDriveDrawSocketApp.exe advanceframe " .. background .. " " .. forward)
end

local function setBackgroundColour(colour)
	os.execute(os.getenv("MegaDriveDrawSocketAppLocation") .. "\\MegaDriveDrawSocketApp.exe setbackgroundcolour " .. colour)
end

local dlg = Dialog("Mega Drive")
dlg
  :separator{ text="Commands for the Mega Drive: Foreground" }
  :button{text="Clear Scene",onclick=function() os.execute(os.getenv("MegaDriveDrawSocketAppLocation") .. "\\MegaDriveDrawSocketApp.exe clear 0") end}
  :button{text="Send Scene",onclick=function() sendScene(0) end}
  :button{text="Update Frame",onclick=function() sendFrame(0) end}
  :button{text="Update Palette",onclick=function() sendPalette(1) end}
  :newrow()
  :button{text="Frame <",onclick=function() advanceFrame(0, 0) end}
  :button{text="Toggle Animation",onclick=function() os.execute(os.getenv("MegaDriveDrawSocketAppLocation") .. "\\MegaDriveDrawSocketApp.exe toggleanimation 0") end}
  :button{text="Frame >",onclick=function() advanceFrame(0, 1) end}
  :separator{ text="Commands for the Mega Drive: Background" }
  :button{text="Clear Scene",onclick=function() os.execute(os.getenv("MegaDriveDrawSocketAppLocation") .. "\\MegaDriveDrawSocketApp.exe clear 1") end}
  :button{text="Send Scene",onclick=function() sendScene(1) end}
  :button{text="Update Frame",onclick=function() sendFrame(1) end}
  :button{text="Update Palette",onclick=function() sendPalette(2) end}
  :newrow()
  :button{text="Frame <",onclick=function() advanceFrame(1, 0) end}
  :button{text="Toggle Animation",onclick=function() os.execute(os.getenv("MegaDriveDrawSocketAppLocation") .. "\\MegaDriveDrawSocketApp.exe toggleanimation 1") end}
  :button{text="Frame >",onclick=function() advanceFrame(1, 1) end}
  :separator{ text="Background Colour" }
  :button{text="Black",onclick=function() setBackgroundColour(0) end}
  :button{text="Dark Grey",onclick=function() setBackgroundColour(1) end}
  :button{text="Grey",onclick=function() setBackgroundColour(2) end}
  :button{text="Light Grey",onclick=function() setBackgroundColour(3) end}
  :separator{ text="Aseprite Commands" }
  :button{text="Crush Palette",onclick=function() crushPalette() end}
  :show{wait=false}